//-----------------------------------------------------------------------------
// File: Meshes.cpp
//
// Desc: For advanced geometry, most apps will prefer to load pre-authored
//       Meshes from a file. Fortunately, when using Meshes, D3DX does most of
//       the work for this, parsing a geometry file and creating vertx buffers
//       (and index buffers) for us. This tutorial shows how to use a D3DXMESH
//       object, including loading it from a file and rendering it. One thing
//       D3DX does not handle for us is the materials and textures for a mesh,
//       so note that we have to handle those manually.
//
//       Note: one advanced (but nice) feature that we don't show here is that
//       when cloning a mesh we can specify the FVF. So, regardless of how the
//       mesh was authored, we can add/remove normals, add more texture
//       coordinate sets (for multi-texturing), etc.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#include <Windows.h>
#include <mmsystem.h>
#include <d3dx9.h>
#pragma warning( disable : 4996 ) // disable deprecated warning 
#include <strsafe.h>
#pragma warning( default : 4996 ) 

#include <d3dx9mesh.h>

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D9             g_pD3D           = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice     = NULL; // Our rendering device

LPD3DXMESH              g_pMesh          = NULL; // Our mesh object in sysmem
D3DMATERIAL9*           g_pMeshMaterials = NULL; // Materials for our mesh
LPDIRECT3DTEXTURE9*     g_pMeshTextures  = NULL; // Textures for our mesh
DWORD                   g_dwNumMaterials = 0L;   // Number of mesh materials

D3DXMATRIXA16 g_MatProj;
D3DXMATRIXA16 g_MatView;
D3DXMATRIXA16 g_MatWorld;

LPD3DXEFFECT g_pEffect = NULL;

D3DXVECTOR3 g_vEyePt;

LPDIRECT3DSURFACE9 g_pDirect3DSurfaceBack = NULL;

// nCCgt̃_O
LPDIRECT3DTEXTURE9 g_pDirect3DTextureFirst = NULL;
LPDIRECT3DSURFACE9 g_pDirect3DSurfaceFirst = NULL;

// nCCgo & OA
LPDIRECT3DTEXTURE9 g_pDirect3DTextureHighlight[2] = { NULL, NULL };
LPDIRECT3DSURFACE9 g_pDirect3DSurfaceHighlight[2] = { NULL, NULL };

// ԕ̃[pX
LPDIRECT3DTEXTURE9 g_pDirect3DTextureAccumulation = NULL;
LPDIRECT3DSURFACE9 g_pDirect3DSurfaceAccumulation = NULL;

LPDIRECT3DSURFACE9 g_pZStencilSurface = NULL;

// _ݒ
struct VERTEX_XYZ_RHW_COLOR_UV
{
	float x, y, z;
	float rhw;
	D3DCOLOR color;
	float u, v;
};

#define	D3DFVF_XYZ_RHW_COLOR_UV	( D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1 )

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice. Since we are now
    // using more complex geometry, we will create a device with a zbuffer.
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.BackBufferWidth  = 512;
	d3dpp.BackBufferHeight = 512;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
		if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hWnd,
										D3DCREATE_SOFTWARE_VERTEXPROCESSING,
										&d3dpp, &g_pd3dDevice ) ) ){
	        return E_FAIL;
		}
    }

    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
	g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2 ,D3DTA_DIFFUSE );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

	g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );

	LPD3DXBUFFER pCompilationErrors;
    if( FAILED ( 
		D3DXCreateEffectFromFile( g_pd3dDevice, "hlsl.fx", NULL, NULL, D3DXSHADER_DEBUG/* | D3DXSHADER_SKIPOPTIMIZATION*/, NULL, &g_pEffect, &pCompilationErrors ) )
	){
		MessageBox( NULL, (LPCSTR)pCompilationErrors->GetBufferPointer(), "Error", MB_OK );
        return E_FAIL;
	}

	g_pEffect->OnResetDevice();


	// eT[tF[X
	g_pd3dDevice->GetRenderTarget( 0, &g_pDirect3DSurfaceBack );

	D3DSURFACE_DESC Desc;
	g_pDirect3DSurfaceBack->GetDesc( &Desc );

	g_pd3dDevice->CreateTexture( 512, 512, 1, D3DUSAGE_RENDERTARGET,
		Desc.Format, D3DPOOL_DEFAULT, &g_pDirect3DTextureFirst, NULL );

	g_pDirect3DTextureFirst->GetSurfaceLevel( 0, &g_pDirect3DSurfaceFirst );

	g_pd3dDevice->CreateDepthStencilSurface( 512, 512,
		D3DFMT_D16, D3DMULTISAMPLE_NONE, 0, FALSE, &g_pZStencilSurface, NULL );

	for( int i = 0; i < 2; ++i ){
		g_pd3dDevice->CreateTexture( 64, 64, 1, D3DUSAGE_RENDERTARGET,
			Desc.Format, D3DPOOL_DEFAULT, &g_pDirect3DTextureHighlight[i], NULL );

		g_pDirect3DTextureHighlight[i]->GetSurfaceLevel( 0, &g_pDirect3DSurfaceHighlight[i] );
	}

	g_pd3dDevice->CreateTexture( 64, 64, 1, D3DUSAGE_RENDERTARGET,
		Desc.Format, D3DPOOL_DEFAULT, &g_pDirect3DTextureAccumulation, NULL );

	g_pDirect3DTextureAccumulation->GetSurfaceLevel( 0, &g_pDirect3DSurfaceAccumulation );

	g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceAccumulation );

	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
						D3DCOLOR_XRGB( 0, 0, 0 ), 1.0f, 0 );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Load the mesh and build the material and texture arrays
//-----------------------------------------------------------------------------
HRESULT InitGeometry()
{
    LPD3DXBUFFER pD3DXMtrlBuffer;

    // Load the mesh from the specified file
    if( FAILED( D3DXLoadMeshFromX( "test.x", D3DXMESH_SYSTEMMEM, 
                                   g_pd3dDevice, NULL, 
                                   &pD3DXMtrlBuffer, NULL, &g_dwNumMaterials, 
                                   &g_pMesh ) ) )
    {
        MessageBox(NULL, "Could not find test.x", "Meshes.exe", MB_OK);
        return E_FAIL;
    }

    // We need to extract the material properties and texture names from the 
    // pD3DXMtrlBuffer
    D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
    g_pMeshMaterials = new D3DMATERIAL9[g_dwNumMaterials];
    if( g_pMeshMaterials == NULL )
        return E_OUTOFMEMORY;
    g_pMeshTextures  = new LPDIRECT3DTEXTURE9[g_dwNumMaterials];
    if( g_pMeshTextures == NULL )
        return E_OUTOFMEMORY;

    for( DWORD i=0; i<g_dwNumMaterials; i++ )
    {
        // Copy the material
        g_pMeshMaterials[i] = d3dxMaterials[i].MatD3D;

        // Set the ambient color for the material (D3DX does not do this)
        g_pMeshMaterials[i].Ambient = g_pMeshMaterials[i].Diffuse;

        g_pMeshTextures[i] = NULL;
        if( d3dxMaterials[i].pTextureFilename != NULL && 
            lstrlen(d3dxMaterials[i].pTextureFilename) > 0 )
        {
            // Create the texture
            if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, 
                                                d3dxMaterials[i].pTextureFilename, 
                                                &g_pMeshTextures[i] ) ) )
            {
                MessageBox(NULL, "Could not find texture map", "Meshes.exe", MB_OK);
            }
        }
    }

    // Done with the material buffer
    pD3DXMtrlBuffer->Release();

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{
	if( g_pZStencilSurface ) g_pZStencilSurface->Release();

	if( g_pDirect3DTextureAccumulation ) g_pDirect3DTextureAccumulation->Release();
	if( g_pDirect3DSurfaceAccumulation ) g_pDirect3DSurfaceAccumulation->Release();
	for( int i = 0; i < 2; ++i ){
		if( g_pDirect3DTextureHighlight[i] ) g_pDirect3DTextureHighlight[i]->Release();
		if( g_pDirect3DSurfaceHighlight[i] ) g_pDirect3DSurfaceHighlight[i]->Release();
	}
	if( g_pDirect3DTextureFirst ) g_pDirect3DTextureFirst->Release();
	if( g_pDirect3DSurfaceFirst ) g_pDirect3DSurfaceFirst->Release();

    if( g_pMeshMaterials != NULL ) 
        delete[] g_pMeshMaterials;

    if( g_pMeshTextures )
    {
        for( DWORD i = 0; i < g_dwNumMaterials; i++ )
        {
            if( g_pMeshTextures[i] )
                g_pMeshTextures[i]->Release();
        }
        delete[] g_pMeshTextures;
    }
    if( g_pMesh != NULL )
        g_pMesh->Release();
    
    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();
}



//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------
VOID SetupMatrices()
{
    // Set up world matrix
	D3DXMATRIXA16 matX, matY, matZ;

    D3DXMatrixRotationX( &matX, timeGetTime() / 1000.0f );
    D3DXMatrixRotationY( &matY, timeGetTime() / 3000.0f );
    D3DXMatrixRotationZ( &matZ, timeGetTime() / 5000.0f );
	g_MatWorld = matX * matY * matZ;

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the 
    // origin, and define "up" to be in the y-direction.
    g_vEyePt = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
    D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &g_MatView, &g_vEyePt, &vLookatPt, &vUpVec );

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMatrixPerspectiveFovLH( &g_MatProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f );
}




//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
    // Begin the scene
    if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
    {
		// HDR_O
		g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
		g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

        // Setup the world, view, and projection matrices
        SetupMatrices();

        UINT nPasses;
        UINT iPass;

		g_pEffect->SetTechnique( g_pEffect->GetTechniqueByName( "HDRShader" ) );
		
        g_pEffect->SetMatrix( "MatrixW", &g_MatWorld );

		D3DXMATRIX m = g_MatWorld * g_MatView * g_MatProj;
        g_pEffect->SetMatrix( "MatrixWVP", &m );

		g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

		g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceFirst );
		g_pd3dDevice->SetDepthStencilSurface( g_pZStencilSurface );

		g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
							D3DCOLOR_XRGB( 0, 0, 0 ), 1.0f, 0 );

		g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

		g_pEffect->Begin( &nPasses, 0 );

		// Cg̐ݒ
		D3DCOLORVALUE LightDiffuseColor	= { 1.0f, 1.0f, 1.0f, 1.0f };
		D3DCOLORVALUE LightAmbientColor	= { 0.0f, 0.0f, 0.0f, 1.0f };
		g_pEffect->SetValue( "LightDiffuseColor", &LightDiffuseColor, sizeof( D3DCOLORVALUE ) );
		g_pEffect->SetValue( "LightAmbientColor", &LightAmbientColor, sizeof( D3DCOLORVALUE ) );

		D3DXVECTOR3 LightDirection = D3DXVECTOR3( 1.0f, -1.0f, 1.0f ); // [hW
		D3DXVec3Normalize ( &LightDirection, &LightDirection );

		g_pEffect->SetValue( "LightDirection", &LightDirection, sizeof( D3DXVECTOR3 ) );

		D3DXVECTOR3 EyeDirection; // [hW
		D3DXVec3Normalize ( &EyeDirection, &g_vEyePt );

		g_pEffect->SetValue( "EyeDirection", &EyeDirection, sizeof( D3DXVECTOR3 ) );

		for( iPass = 0; iPass < nPasses; iPass ++ ){
			g_pEffect->BeginPass( iPass );

			// Meshes are divided into subsets, one for each material. Render them in
			// a loop
			for( DWORD i=0; i<g_dwNumMaterials; i++ )
			{
				// Set the material and texture for this subset
				if( g_pMeshTextures[i] ) g_pEffect->SetTexture( "MeshTexture", g_pMeshTextures[i] );

				g_pEffect->SetValue( "MaterialAmbientColor", &g_pMeshMaterials[i].Ambient, sizeof( D3DCOLORVALUE ) );
				g_pEffect->SetValue( "MaterialDiffuseColor", &g_pMeshMaterials[i].Diffuse, sizeof( D3DCOLORVALUE ) );

				g_pEffect->CommitChanges();

				// Draw the mesh subset
				g_pMesh->DrawSubset( i );
			}

			g_pEffect->EndPass();
		}
		g_pEffect->End();

		// nCCgo
		{
			g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
			g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

			g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceHighlight[0] );

			VERTEX_XYZ_RHW_COLOR_UV Vertex[4] = {
				{  0,  0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 0 },
				{ 64,  0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 0 },
				{ 64, 64, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 1 },
				{  0, 64, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 1 }
			};

			// eNX`ݒ
			g_pEffect->SetTexture( "MeshTexture", g_pDirect3DTextureFirst );

			// _tH[}bg̐ݒ
			g_pd3dDevice->SetFVF( D3DFVF_XYZ_RHW_COLOR_UV );

			g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			g_pEffect->SetTechnique( g_pEffect->GetTechniqueByName( "HighlightShader" ) );

			g_pEffect->Begin( &nPasses, 0 );
			for( iPass = 0; iPass < nPasses; iPass ++ ){
				g_pEffect->BeginPass( iPass );
				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
				g_pEffect->EndPass();
			}
			g_pEffect->End();
		}

		// nCCgڂ
		for( int i = 0; i < 5; ++i ){
			if( i % 2 == 0 ) g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceHighlight[1] );
			else g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceHighlight[0] );

			VERTEX_XYZ_RHW_COLOR_UV Vertex[4] = {
				{  0,  0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0+0.5f/64, 0+0.5f/64 },
				{ 64,  0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1+0.5f/64, 0+0.5f/64 },
				{ 64, 64, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1+0.5f/64, 1+0.5f/64 },
				{  0, 64, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0+0.5f/64, 1+0.5f/64 }
			};

			// eNX`ݒ
			if( i % 2 == 0 ) g_pEffect->SetTexture( "MeshTexture", g_pDirect3DTextureHighlight[0] );
			else g_pEffect->SetTexture( "MeshTexture", g_pDirect3DTextureHighlight[1] );

			// _tH[}bg̐ݒ
			g_pd3dDevice->SetFVF( D3DFVF_XYZ_RHW_COLOR_UV );

			g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			g_pEffect->SetTechnique( g_pEffect->GetTechniqueByName( "LowpassShader" ) );

			g_pEffect->Begin( &nPasses, 0 );
			for( iPass = 0; iPass < nPasses; iPass ++ ){
				g_pEffect->BeginPass( iPass );
				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
				g_pEffect->EndPass();
			}
			g_pEffect->End();
		}

		// nCCg\^ɉH
		{
			g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_ONE );
			g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );

			g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceHighlight[0] );
			g_pd3dDevice->SetTexture( 0, g_pDirect3DTextureHighlight[1] );

			for( int i = 0; i < 15; i = i + 3 ){
				int R = 255 - i * 20;
				int G = 255 - i * 15;
				int B = 255 - i * 10;

				float h = (float)i * 0.8f;
				float v = (float)i;

				VERTEX_XYZ_RHW_COLOR_UV Vertex_LU[4] = {
					{  0 - h,  0 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 0 },
					{ 64 - h,  0 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 0 },
					{ 64 - h, 64 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 1 },
					{  0 - h, 64 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 1 }
				};
				VERTEX_XYZ_RHW_COLOR_UV Vertex_RU[4] = {
					{  0 + h,  0 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 0 },
					{ 64 + h,  0 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 0 },
					{ 64 + h, 64 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 1 },
					{  0 + h, 64 - v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 1 }
				};
				VERTEX_XYZ_RHW_COLOR_UV Vertex_LD[4] = {
					{  0 - h,  0 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 0 },
					{ 64 - h,  0 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 0 },
					{ 64 - h, 64 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 1 },
					{  0 - h, 64 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 1 }
				};
				VERTEX_XYZ_RHW_COLOR_UV Vertex_RD[4] = {
					{  0 + h,  0 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 0 },
					{ 64 + h,  0 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 0 },
					{ 64 + h, 64 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 1, 1 },
					{  0 + h, 64 + v, 0, 1.0f, D3DCOLOR_ARGB( 255, R, G, B ), 0, 1 }
				};

				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex_LU, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex_RU, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex_LD, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
				g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex_RD, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
			}
		}

		// ԕ̃[pX
		{
			g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
			g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

			g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceAccumulation );

			VERTEX_XYZ_RHW_COLOR_UV Vertex[4] = {
				{  0,  0, 0, 1.0f, D3DCOLOR_ARGB( 96, 255, 255, 255 ), 0, 0 },
				{ 64,  0, 0, 1.0f, D3DCOLOR_ARGB( 96, 255, 255, 255 ), 1, 0 },
				{ 64, 64, 0, 1.0f, D3DCOLOR_ARGB( 96, 255, 255, 255 ), 1, 1 },
				{  0, 64, 0, 1.0f, D3DCOLOR_ARGB( 96, 255, 255, 255 ), 0, 1 }
			};

			// eNX`ݒ
			g_pd3dDevice->SetTexture( 0, g_pDirect3DTextureHighlight[0] );

			// _tH[}bg̐ݒ
			g_pd3dDevice->SetFVF( D3DFVF_XYZ_RHW_COLOR_UV );

			g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
		}

		// ŌɃOA
		{
			g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
			g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

			g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceBack );

			VERTEX_XYZ_RHW_COLOR_UV Vertex[4] = {
				{   0,   0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 0 },
				{ 512,   0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 0 },
				{ 512, 512, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 1 },
				{   0, 512, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 1 }
			};

			// eNX`ݒ
			g_pd3dDevice->SetTexture( 0, g_pDirect3DTextureFirst );

			// _tH[}bg̐ݒ
			g_pd3dDevice->SetFVF( D3DFVF_XYZ_RHW_COLOR_UV );

			g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
		}

		// OA
		{
			g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_ONE );
			g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );

			g_pd3dDevice->SetRenderTarget( 0, g_pDirect3DSurfaceBack );

			VERTEX_XYZ_RHW_COLOR_UV Vertex[4] = {
				{   0,   0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 0 },
				{ 512,   0, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 0 },
				{ 512, 512, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 1, 1 },
				{   0, 512, 0, 1.0f, D3DCOLOR_ARGB( 255, 255, 255, 255 ), 0, 1 }
			};

			// eNX`ݒ
//			g_pd3dDevice->SetTexture( 0, g_pDirect3DTextureHighlight[0] );
			g_pd3dDevice->SetTexture( 0, g_pDirect3DTextureAccumulation );
	
			// _tH[}bg̐ݒ
			g_pd3dDevice->SetFVF( D3DFVF_XYZ_RHW_COLOR_UV );

			g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			g_pd3dDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( VERTEX_XYZ_RHW_COLOR_UV ) );
		}

        // End the scene
        g_pd3dDevice->EndScene();
    }

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            Cleanup();
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "D3D Tutorial", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "D3D Tutorial", "D3D Tutorial 06: Meshes", 
                              WS_OVERLAPPEDWINDOW, 100, 100, 512, 512,
                              NULL, NULL, wc.hInstance, NULL );

    // Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) )
    { 
        // Create the scene geometry
        if( SUCCEEDED( InitGeometry() ) )
        {
            // Show the window
            ShowWindow( hWnd, SW_SHOWDEFAULT );
            UpdateWindow( hWnd );

            // Enter the message loop
            MSG msg; 
            ZeroMemory( &msg, sizeof(msg) );
            while( msg.message!=WM_QUIT )
            {
                if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
                {
                    TranslateMessage( &msg );
                    DispatchMessage( &msg );
                }
                else
                    Render();
            }
        }
    }

    UnregisterClass( "D3D Tutorial", wc.hInstance );
    return 0;
}



